/*
 * Project name:
     RTC_Write (Demonstration on working with the RTC Module and Software I2C routines)
 * Copyright:
     (c) Mikroelektronika, 2005-2010.
 * Revision History:
     20080930:
       - initial release;
 * Description:
      This project is simple demonstration how to set date and time on PCF8583
      RTC (real-time clock).
 * Test configuration:
     MCU:             ATmega1280
     Oscillator:      External Clock, 8.0000 MHz
     Ext. Modules:    ac:RTC_Board on PORTD
     SW:              mikroC PRO for AVR
                      http://www.mikroe.com/eng/products/view/228/mikroc-pro-for-avr/
 * NOTES:
     - I2C communication lines should be connected to pull-up resistors.
     - Turn off LEDs connected to I2C communication lines.
     - In order to use the example, address pin A0 of PCF8583 must be set to 0V.
       (on mikroElektronika's RTC module this is done by default)
*/
// char seconds, minutes, hours, day, month, year;    // Global date/time variables
// Software I2C connections
sbit Soft_I2C_Scl_Output    at PORTD0_bit;
sbit Soft_I2C_Sda_Output    at PORTD1_bit;
sbit Soft_I2C_Scl_Input     at PIND0_bit;
sbit Soft_I2C_Sda_Input     at PIND1_bit;
sbit Soft_I2C_Scl_Direction at DDD0_bit;
sbit Soft_I2C_Sda_Direction at DDD1_bit;
// End Software I2C connections
// void Write_Time();

void Write_Time(unsigned char c_hours, unsigned char c_minutes, unsigned char c_seconds, unsigned char c_date, unsigned char c_month, unsigned char c_year, unsigned char c_wday) {

// c_year = c_year;
c_year = (c_year % 4) << 6;
c_wday = (c_wday) << 5;
   Soft_I2C_Start();      // Issue start signal
   Soft_I2C_Write(0xA0);  // Address PCF8583 datasheet
   Soft_I2C_Write(0);     // Start from address 0 (configuration memory location)
   Soft_I2C_Write(0x80);  // Write 0x80 to configuration memory location (pause counter...)
   Soft_I2C_Write(0);     // Write 0 to cents memory location
   Soft_I2C_Write(c_seconds);     // Write 0 to seconds memory location
   Soft_I2C_Write(c_minutes);  // Write 0x30 to minutes memory location
   Soft_I2C_Write(c_hours);  // Write 0x9 to hours memory location
   Soft_I2C_Write(c_date | c_year);  // Write 0x24 to date/year memory location
   Soft_I2C_Write(c_month | c_wday);  // Write 0x08 to weekday/month memory location
   Soft_I2C_Stop();       // Issue stop signal

   Soft_I2C_Start();      // Issue start signal
   Soft_I2C_Write(0xA0);  // Address PCF8583
   Soft_I2C_Write(0);     // Start from address 0
   Soft_I2C_Write(0);     // Write 0 to configuration memory location (enable counting)
   Soft_I2C_Stop();       // Issue stop signal
}

void main() {
// write

   Delay_ms(100);
   Soft_I2C_Init();       // Initialize full master mode
   Write_Time(0x13, 0x10, 0x00, 0x16, 0x10, 12, 1);
   //   Write   13:10:00  16 Oktober 2012 Tuesday
}