#include "atmegamp3_objects.h"
#include "atmegamp3_resources.h"

//--------------------- User code ---------------------//
#include "resources.h"

// Software I2C connections
sbit Soft_I2C_Scl_Output    at PORTD0_bit;
sbit Soft_I2C_Sda_Output    at PORTD1_bit;
sbit Soft_I2C_Scl_Input     at PIND0_bit;
sbit Soft_I2C_Sda_Input     at PIND1_bit;
sbit Soft_I2C_Scl_Direction at DDD0_bit;
sbit Soft_I2C_Sda_Direction at DDD1_bit;
// End Software I2C connections

char sek, seconds, minutes, hours, day, weekday, month, year;    // Global date/time variables
char second[16], minute[16], hour[16], days[16], weekdays[16];

unsigned long file_size;
const BYTES_2_WRITE = 32;
const BUFFER_SIZE = 512;
char mp3_buffer[BUFFER_SIZE];
char volume_left, volume_right;

// Functions
void MP3_Init(void);
void MP3_Start(void);
void MP3_Test(char *test);

//- Imported from MP3_DRIVER.H --------------------------------------------------------------------
extern const char WRITE_CODE;
extern const char READ_CODE;

extern const char SCI_BASE_ADDR;
extern const char SCI_MODE_ADDR;
extern const char SCI_STATUS_ADDR;
extern const char SCI_BASS_ADDR;
extern const char SCI_CLOCKF_ADDR;
extern const char SCI_DECODE_TIME_ADDR;
extern const char SCI_AUDATA_ADDR;
extern const char SCI_WRAM_ADDR;
extern const char SCI_WRAMADDR_ADDR;
extern const char SCI_HDAT0_ADDR;
extern const char SCI_HDAT1_ADDR;
extern const char SCI_AIADDR_ADDR;
extern const char SCI_VOL_ADDR;
extern const char SCI_AICTRL0_ADDR;
extern const char SCI_AICTRL1_ADDR;
extern const char SCI_AICTRL2_ADDR;
extern const char SCI_AICTRL3_ADDR;

extern void MP3_SCI_Write(char address, unsigned int data_in);
extern void MP3_SCI_Read(char start_address, char words_count, unsigned int *data_buffer);
extern void MP3_SDI_Write(char data_);
extern void MP3_SDI_Write_32(char *data_);
extern void MP3_SDI_Write_256(char *data_);
extern void MP3_Set_Volume(char left,char right);

// !!!! This is only for testing proposes, remove for normal use !!!!
unsigned char ucMP3_run_test;
// !!!! This is only for testing proposes, remove for normal use !!!!

/**************************************************************************************************
* End of File
**************************************************************************************************/
/**************************************************************************************************
* CODEC V1053E connections
**************************************************************************************************/
sbit MP3_CS                        at PORTB7_bit;
sbit MP3_RST                       at PORTH7_bit;
sbit BSYNC                         at PORTF3_bit;
sbit DREQ                          at  PING3_bit;

sbit MP3_CS_Direction              at   DDB7_bit;
sbit MP3_RST_Direction             at   DDH7_bit;
sbit BSYNC_Direction               at   DDF3_bit;
sbit DREQ_Direction                at   DDG3_bit;

/**************************************************************************************************
* CODEC V1053E connections
**************************************************************************************************/
extern sfr sbit Mmc_Chip_Select;
extern sfr sbit MP3_CS;
extern sfr sbit MP3_RST;
extern sfr sbit BSYNC;
extern sfr sbit DREQ;

extern sfr sbit Mmc_Chip_Select_Direction;
extern sfr sbit MP3_CS_Direction;
extern sfr sbit MP3_RST_Direction;
extern sfr sbit BSYNC_Direction;
extern sfr sbit DREQ_Direction;

/**************************************************************************************************
* Function MP3_SCI_Write()
* -------------------------------------------------------------------------------------------------
* Overview: Function writes one byte to MP3 SCI
* Input: register address in codec, data
* Output: Nothing
**************************************************************************************************/
void MP3_SCI_Write(char address, unsigned int data_in) {
  BSYNC = 1;

  MP3_CS = 0;                    // select MP3 SCI
  SPI1_Write(WRITE_CODE);
  SPI1_Write(address);
  SPI1_Write(Hi(data_in));       // high byte
  SPI1_Write(Lo(data_in));       // low byte
  MP3_CS = 1;                    // deselect MP3 SCI
  while (DREQ == 0)              // wait until DREQ becomes 1, see MP3 codec datasheet, Serial Protocol for SCI
    ;
}
/**************************************************************************************************
* Function MP3_SCI_Read()
* -------------------------------------------------------------------------------------------------
* Overview: Function reads words_count words from MP3 SCI
* Input: start address, word count to be read
* Output: words are stored to data_buffer
**************************************************************************************************/
void MP3_SCI_Read(char start_address, char words_count, unsigned int *data_buffer) {
  unsigned int temp;

  MP3_CS = 0;                    // select MP3 SCI
  SPI1_Write(READ_CODE);
  SPI1_Write(start_address);

  while (words_count--) {        // read words_count words byte per byte
    temp = SPI1_Read(0);
    temp <<= 8;
    temp += SPI1_Read(0);
    *(data_buffer++) = temp;
  }
  MP3_CS = 1;                    // deselect MP3 SCI
  while (DREQ == 0)              // wait until DREQ becomes 1, see MP3 codec datasheet, Serial Protocol for SCI
    ;
}


/**************************************************************************************************
* Function MP3_SDI_Write()
* -------------------------------------------------------------------------------------------------
* Overview: Function write one byte to MP3 SDI
* Input: data to be writed
* Output: Nothing
**************************************************************************************************/
void MP3_SDI_Write(char data_) {

  MP3_CS = 1;
  BSYNC = 0;

  while (DREQ == 0);             // wait until DREQ becomes 1, see MP3 codec datasheet, Serial Protocol for SCI

  SPI1_Write(data_);
  BSYNC = 1;
}


/**************************************************************************************************
* Function MP3_SDI_Write_32
* -------------------------------------------------------------------------------------------------
* Overview: Function Write 32 bytes to MP3 SDI
* Input: data buffer
* Output: Nothing
**************************************************************************************************/
void MP3_SDI_Write_32(char *data_) {
  char i;

  MP3_CS = 1;
  BSYNC = 0;

  while (DREQ == 0);             // wait until DREQ becomes 1, see MP3 codec datasheet, Serial Protocol for SCI

  for (i=0; i<32; i++)
  SPI1_Write(data_[i]);
  BSYNC = 1;
}

/**************************************************************************************************
* Function MP3_Set_Volume()
* -------------------------------------------------------------------------------------------------
* Overview: Function set volume on the left and right channel
* Input: left channel volume, right channel volume
* Output: Nothing
**************************************************************************************************/
void MP3_Set_Volume(char left, char right) {
  unsigned int volume;

  volume = (left<<8) + right;             // calculate value
  MP3_SCI_Write(SCI_VOL_ADDR, volume);    // Write value to VOL register
}

/**************************************************************************************************
* End of File
**************************************************************************************************/

/**************************************************************************************************
* MMC Chip Select connection
**************************************************************************************************/
// sbit Mmc_Chip_Select               at PORTJ3_bit;
// sbit Mmc_Chip_Select_Direction     at   DDJ3_bit;

/**************************************************************************************************
* Serial Flash Chip Select connection
**************************************************************************************************/
sbit CS_Serial_Flash_bit           at PORTG4_bit;
sbit CS_Serial_Flash_Direction_bit at   DDG4_bit;

// VS1053E constants
const char WRITE_CODE           = 0x02;
const char READ_CODE            = 0x03;

const char SCI_BASE_ADDR        = 0x00;
const char SCI_MODE_ADDR        = 0x00;
const char SCI_STATUS_ADDR      = 0x01;
const char SCI_BASS_ADDR        = 0x02;
const char SCI_CLOCKF_ADDR      = 0x03;
const char SCI_DECODE_TIME_ADDR = 0x04;
const char SCI_AUDATA_ADDR      = 0x05;
const char SCI_WRAM_ADDR        = 0x06;
const char SCI_WRAMADDR_ADDR    = 0x07;
const char SCI_HDAT0_ADDR       = 0x08;
const char SCI_HDAT1_ADDR       = 0x09;
const char SCI_AIADDR_ADDR      = 0x0A;
const char SCI_VOL_ADDR         = 0x0B;
const char SCI_AICTRL0_ADDR     = 0x0C;
const char SCI_AICTRL1_ADDR     = 0x0D;
const char SCI_AICTRL2_ADDR     = 0x0E;
const char SCI_AICTRL3_ADDR     = 0x0F;

// Writes one byte to MP3 SCI
void MP3_SCI_Write(char address, unsigned int data_in);
// Reads words_count words from MP3 SCI
void MP3_SCI_Read(char start_address, char words_count, unsigned int *data_buffer);
// Write one byte to MP3 SDI
void MP3_SDI_Write(char data_);
// Write 32 bytes to MP3 SDI
void MP3_SDI_Write_32(char *data_);
// Set volume
void MP3_Set_Volume(char left, char right);

/**************************************************************************************************
* End of File
**************************************************************************************************/


/**************************************************************************************************
* Function MC3_Init()
* -------------------------------------------------------------------------------------------------
* Overview: Function Initialize MP3 VS1053B codec
* Input: Nothing
* Output: Nothing
**************************************************************************************************/

void MP3_Init(void)
{
  BSYNC = 1;
  MP3_CS = 1;

  // Hardware reset
  MP3_RST = 0;
  Delay_ms(10);
  MP3_RST = 1;

  while (DREQ == 0);

  MP3_SCI_Write(SCI_MODE_ADDR, 0x0800);
  MP3_SCI_Write(SCI_BASS_ADDR, 0x7A00);
  MP3_SCI_Write(SCI_CLOCKF_ADDR, 0x2000);   // default 12 288 000 Hz

  // Maximum volume is 0x00 and total silence is 0xFE.
  volume_left  = 0; //0x3F;
  volume_right = 0; //0x3F;
  MP3_Set_Volume(volume_left, volume_right);
}

/**************************************************************************************************
* Function MP3_Start()
* -------------------------------------------------------------------------------------------------
* Overview: Function Initialize SPI to communicate with MP3 codec
* Input: Nothing
* Output: Nothing
**************************************************************************************************/
void MP3_Start(void)
{
  // Disable other peripheral modules on the same SPI bus
  CS_Serial_Flash_Direction_bit = 1;
  CS_Serial_Flash_bit = 1;             // Disable Serial Flash module

  MP3_CS_Direction  = 1;               // Configure MP3_CS as output
  MP3_CS            = 1;               // Deselect MP3_CS
  MP3_RST_Direction = 1;               // Configure MP3_RST as output
  MP3_RST           = 1;               // Set MP3_RST pin

  DREQ_Direction    = 0;               // Configure DREQ as input
  BSYNC_Direction   = 1;               // Configure BSYNC as output
  BSYNC             = 0;               // Clear BSYNC
  BSYNC             = 1;               // Clear BSYNC

  // Initialize SPIC module
  // master_mode                   = _SPI_MASTER
  // Clock rate                    = _SPI_FCY_DIV16
  // SPI clock polarity and phase  = _SPI_CLK_LO_LEADING
  SPI1_Init_Advanced(_SPI_MASTER, _SPI_FCY_DIV4, _SPI_CLK_LO_LEADING);

  TFT_16bit_Set_Pen(CL_WHITE_16bit, 1);
  TFT_16bit_Set_Brush(1, CL_WHITE_16bit, 0, 0, 0, 0);


  MP3_Init();
  Delay_ms(1000);
}

/**************************************************************************************************
* Function MP3_Test()
* -------------------------------------------------------------------------------------------------
* Overview: Function run the MP3 test
* Input: Nothing
* Output: test status: 0 - skiped; 1 - pass; 2 - fail
**************************************************************************************************/
void MP3_Test(char *test)
{
unsigned long i;
unsigned long roma;
roma=0;

  // TFT_16bit_Fill_Screen(CL_Black_16bit);
  TFT_16bit_Set_Pen(CL_WHITE_16bit, 1);
  // TFT_16bit_Line(20, 220, 300, 220);
  // TFT_16bit_Set_Font(&HandelGothic_BT21x22_Regular, CL_RED_16bit, FO_HORIZONTAL_16bit);
  // TFT_16bit_Set_Font(&Verdana12x13_Regular, CL_WHITE_16bit, FO_HORIZONTAL_16bit);
  TFT_16bit_LIne(20,  46, 300,  46);
  TFT_16bit_Set_Font(&Lucida_Console_KOI89x14_Bold, CL_RED_16bit, FO_HORIZONTAL_16bit);
  TFT_16bit_Write_Text("\xF2\xCF\xCD\xC1\xCE", 19, 25);
  TFT_16bit_Set_Font(&TFT_16bit_defaultFont, CL_WHITE_16bit, FO_HORIZONTAL_16bit);
  TFT_16bit_Write_Text("MP3 Player for ATmega", 92, 5);
  TFT_16bit_Set_Font(&TFT_16bit_defaultFont, CL_RED_16bit, FO_HORIZONTAL_16bit);
  TFT_16bit_Write_Text("www.ingineer.ru", 195, 25);


 while(1)
 {

// char mp3_filename[14] = "sound.mp3";   // MP3 file name
 char mp3_filename[25][7] ={ "1.mp3","2.mp3","3.mp3","4.mp3","5.mp3","6.mp3","7.mp3","8.mp3","9.mp3","10.mp3","11.mp3","12.mp3","13.mp3","14.mp3","15.mp3","16.mp3","17.mp3","18.mp3","19.mp3","20.mp3","21.mp3","22.mp3","23.mp3","24.mp3","25.mp3"};
  // Reset error flag
  *test = 0;
  TFT_16bit_Set_Pen(CL_SILVER_16bit, 93);
  TFT_16bit_LIne(26,  126, 161,  126);
  TFT_16bit_Set_Font(&Lucida_Console_KOI89x14_Bold, CL_Black_16bit, FO_HORIZONTAL_16bit);
  TFT_16bit_Write_Text("1. \xEB\xCF\xC4\xC5\xCB \xF7\xC9\xD6\xD5", 30, 80);
  TFT_16bit_Write_Text("2. SD \xF7\xC9\xD6\xD5", 30, 100);

  // TFT_16bit_Set_Font(&TFT_16bit_defaultFont, CL_Black_16bit, FO_HORIZONTAL_16bit);
  if (Mmc_Fat_Init() == 0) {
    if (Mmc_Fat_Assign(&mp3_filename[roma], 0) ) {
      TFT_16bit_Write_Text("3. \xE6\xC1\xCA\xCC:", 30, 120);
      Mmc_Fat_Reset(&file_size);          // Call Reset before file reading,
                                          //   procedure returns size of the file

      TFT_16bit_Write_Text("4. \xF0\xD2\xCF\xC9\xC7\xD2\xD9\xD7\xC1\xC5\xCD", 30, 140);
      // TFT_16bit_Write_Text("_________", 150, 140);
      TFT_16bit_Write_Text(&mp3_filename[roma], 100, 120);

      // send file blocks to MP3 SDI
      while (file_size > BUFFER_SIZE)
      {
        for (i=0; i<BUFFER_SIZE; i++)
        {
          Mmc_Fat_Read(mp3_buffer + i);
        }
        for (i=0; i<BUFFER_SIZE/BYTES_2_WRITE; i++) {
          MP3_SDI_Write_32(mp3_buffer + i*BYTES_2_WRITE);
        }

        file_size -= BUFFER_SIZE;
          sek = seconds;
          Read_Time(); // seconds, minutes, hours, day, month, year
          if (seconds > sek) { print_time(); }
     }

      // send the rest of the file to MP3 SDI
      for (i=0; i<file_size; i++)
      {
        Mmc_Fat_Read(mp3_buffer + i);
      }

      for (i=0; i<file_size; i++)
      {
        MP3_SDI_Write(mp3_buffer[i]);
      }
      // TFT_16bit_Write_Text("_________", 30, 160);
      TFT_16bit_Write_Text("5. \xEB\xCF\xCE\xC5\xC3 \xE6\xC1\xCA\xCC\xC1", 30, 160);


    }
    else {

      TFT_16bit_Write_Text("3. \xE6\xC1\xCA\xCC\xC1 \xCE\xC5\xD4", 30, 120);
      *test = 2;
    }
  }
  else {
    TFT_16bit_Write_Text("3. SD \xCE\xC5 \xD7\xC9\xD6\xD5", 30, 120);
   *test = 2;
  }
  roma++;
  if (roma>25) { roma=0; }
 }
}
/**************************************************************************************************
* main function
**************************************************************************************************/

void mp3() {

  MP3_Start();
  MP3_Test(&ucMP3_run_test);

}
// Event Handlers


void Read_Time() {

  Soft_I2C_Start();               // Issue start signal
  Soft_I2C_Write(0xA0);           // Address PCF8583, see PCF8583 datasheet
  Soft_I2C_Write(2);              // Start from address 2
  Soft_I2C_Start();               // Issue repeated start signal
  Soft_I2C_Write(0xA1);           // Address PCF8583 for reading R/W=1

  seconds = Soft_I2C_Read(1);     // Read seconds byte
  minutes = Soft_I2C_Read(1);     // Read minutes byte
  hours = Soft_I2C_Read(1);       // Read hours byte
  day = Soft_I2C_Read(1);         // Read year/day byte
  month = Soft_I2C_Read(0);       // Read weekday/month byte
  Soft_I2C_Stop();                // Issue stop signal

//-------------------- Formats date and time
  seconds  =  ((seconds & 0xF0) >> 4)*10 + (seconds & 0x0F);  // Transform seconds
  minutes  =  ((minutes & 0xF0) >> 4)*10 + (minutes & 0x0F);  // Transform months
  hours    =  ((hours & 0xF0)  >> 4)*10  + (hours & 0x0F);    // Transform hours
  year     =   (day & 0xC0) >> 6;                             // Transform year
  day      =  ((day & 0x30) >> 4)*10 + (day & 0x0F);          // Transform day
  weekday  =  ((month & 0xE0) >> 5 );                         // Transform weekday  
  month    =  ((month & 0x10)  >> 4)*10 + (month & 0x0F);     // Transform month 

}

// Cyrillic font for RTC
char mes[12][35] = { "\xF1\xCE\xD7\xC1\xD2\xD1","\xE6\xC5\xD7\xD2\xC1\xCC\xD1","\xED\xC1\xD2\xC1","\xE1\xD0\xD2\xC5\xCC\xD1","\xED\xC1\xD1","\xE9\xC0\xCE\xD1","\xE9\xC0\xCC\xD1","\xE1\xD7\xC7\xD5\xD3\xC1","\xF3\xC5\xCE\xD4\xD1\xC2\xD2\xD1","\xEF\xCB\xD4\xD1\xC2\xD2\xD1","\xEE\xCF\xD1\xC2\xD2\xD1","\xE4\xC5\xCB\xC1\xC2\xD2\xD1" }; // months
char ned[7][9] = { "\xF0\xCE","\xF7\xD4","\xF3\xD2","\xFE\xD4","\xF0\xD4","\xF3\xC2","\xF7\xD3" }; //weekdays

void print_time() {
  TFT_16bit_Set_Font(&Lucida_Console_KOI89x14_Bold, CL_Black_16bit, FO_HORIZONTAL_16bit);
  IntToStr(seconds, second);
  IntToStr(minutes, minute);
  IntToStr(hours, hour);
  IntToStr(day, days);
  TFT_16bit_Set_Pen(0x8FB1, 11);
  TFT_16bit_Line(120, 40+6, 210, 40+6);
  TFT_16bit_Write_Text(days, 5, 160);
  TFT_16bit_Write_Text(&mes[month-1], 60, 160);
  TFT_16bit_Write_Text(&ned[weekday], 130, 160);
  TFT_16bit_Write_Text(hour, 100, 40);
  TFT_16bit_Write_Text(":", 145, 40);
  TFT_16bit_Write_Text(minute, 130, 40);
  TFT_16bit_Write_Text(":", 175, 40);
  TFT_16bit_Write_Text(second, 160, 40);
}
//----------------- End of User code ------------------//